@@ -1,4 +1,4 @@
-# $Id: InteractionManager.pm 1 2006-06-01 16:21:45Z erant $
+# $Id: InteractionManager.pm 1376 2007-08-06 15:59:00Z erant $
 #
 # Module  : InteractionManager.pm
 # Purpose : An interaction manager.
@@ -50,11 +50,8 @@ sub work {
 	my @cco_intact_interactors = ();
 
 	# Initialize the parser and load the OBO file
-	#&print_time;
-# 	print $pre_cco_obo_file_name,"\n";
 	my $my_parser = CCO::Parser::OBOParser->new;
 	my $ontology = $my_parser->work($pre_cco_obo_file_name);
-# 	#&print_time;
 
 	# All the interaction ids of the XML files
 	my @all_interactions_EBI_ids = ();
@@ -68,8 +65,6 @@ sub work {
 
 	for my $intact_xml_file(@intact_xml_files){
 		my $XMLIntactParser = CCO::Util::XMLIntactParser->new;
-		#&print_time;
-# 		print $intact_xml_file,"\n";
 		$XMLIntactParser->work($intact_xml_file);
 		my @xml_interactors = @{$XMLIntactParser->interactors()};
 		my @xml_interactions = @{$XMLIntactParser->interactions()};
@@ -83,12 +78,12 @@ sub work {
 			if (!defined (&lookup($interactor_name,\@checked_interactors_ids))){
 				push (@checked_interactors_ids,$interactor_name);
 				if (defined $ontology->get_term_by_xref('UniProt', $interactor_name)){
-# 					print "Interactor in OBO:",$interactor_name,"\n";
 					my @this_interactions = @{&retrieve_interactions($xml_interactor,\@xml_interactions)};
 					for my $this_interaction (@this_interactions){
 						my $interaction_EBI_id = $this_interaction->primaryRef;
 						if (!defined (&lookup($interaction_EBI_id,\@checked_interactions_ids))){
 							my $interaction_for_cco = &merge_interactors($this_interaction,\@xml_interactors);
+							$this_interaction->goodInteractorID($xml_interactor->id);
 							push (@interactions_for_cco,$interaction_for_cco);
 							push (@checked_interactions_ids,$interaction_EBI_id);
 						}
@@ -96,14 +91,12 @@ sub work {
 				}
 			}			
 		}
-# 		#&print_time;
 	}
 	
 	my $all_interactions_size = @all_interactions_EBI_ids;
 	my $cco_interactions_size = @interactions_for_cco;
 # 	print $all_interactions_size," total interactions: ",$cco_interactions_size," interactions chosen \n";
 
-	
 	# Load the ids
 	my $cco_i_taxon_ids_map = CCO::Util::CCO_ID_Term_Map->new($cco_i_taxon_ids);
 	my $cco_b_taxon_ids_map = CCO::Util::CCO_ID_Term_Map->new($cco_b_taxon_ids);
@@ -115,53 +108,83 @@ sub work {
 	# goa file in OBO. Check if the interaction is 
 	# already in the OBO file: if the interaction is in OBO (same xref), 
 	# add the extras; if it isn't, add the interaction altogether with the extras
+	
+	# Generate the ids for interaction types
+	my %cco_y_ids = %{&generate_cco_y_ids()};
+# 	print "========>",$cco_y_ids{'physical interaction'},"\n";
 
-	#&print_time;
 	my @oboed_interactions = ();
 	my @original_obo_interactions = @{$ontology->get_terms("CCO:I.*")};
 	for my $cco_intact_interaction (@interactions_for_cco){
-# 		print $cco_intact_interaction->primaryRef,"\n";
-# 		#&print_time;
 		# does the interaction exist in OBO?
-		#my $obo_interaction_term = &check_term_exists_by_xref_acc($cco_intact_interaction->primaryRef,@original_obo_interactions);
 		my $obo_interaction_term = $ontology->get_term_by_xref('IntAct', $cco_intact_interaction->primaryRef);
 		if (defined $obo_interaction_term){
 			# TODO: check interactors and change them accordingly
-# 			print "Interaction already in OBO: ",$cco_intact_interaction->primaryRef,"\n";
 		}
 		else{
-# 			#&print_time;
 			push(@oboed_interactions,$cco_intact_interaction);
-# 			print "Adding interaction to OBO: ",$cco_intact_interaction->primaryRef,"\n";
-# 			#&print_time;
 			my $OBOed_EBI_interaction = &add_interaction($cco_intact_interaction,$ontology,$cco_i_taxon_ids_map,$cco_i_ids_map);
-# 			#&print_time;
+			# Add interaction type
+			my $interaction_type = $cco_intact_interaction->interactionType;
+			my $CCO_interaction_type = $ontology->get_term_by_name_or_synonym($interaction_type);			
+			if (defined $CCO_interaction_type){
+				&add_rel("is_a", $OBOed_EBI_interaction, $CCO_interaction_type, $ontology);
+			}
+			else{
+				print $interaction_type, " -> NO INTERACTION TYPE FOUND IN CCO!!!\n";
+			}
+			# Add interactors
+			
+			my $good_interactor_is_bait=0;
+			# Get the good interactor and check whether is bait or prey
+			for my $ebi_interactor (@{$cco_intact_interaction->interactors}){
+				if($ebi_interactor->id eq $cco_intact_interaction->goodInteractorID){
+					my $role = &get_role ($ebi_interactor->id,$cco_intact_interaction);
+					if($role eq "bait"){
+						$good_interactor_is_bait=1;
+					}
+					if($role eq "prey"){
+						$good_interactor_is_bait=0;
+					}
+				}
+			}
+			
+
 			for my $ebi_interactor (@{$cco_intact_interaction->interactors}){
 				if((defined $ebi_interactor->ncbiTaxId) and ($ebi_interactor->ncbiTaxId eq $taxon)){ 
 					# Work only with interactors of the given taxon
-					# TODO: this is wrong cause we will miss things like ATP, with no taxon cause it's in every taxon
+					# TODO: this is wrong cause we will miss things like ATP, 
+					#with no taxon cause it's in every taxon
 					# although this is just in a few cases
-# 					#&print_time;
-					#my $obo_interactor_term = $ontology->get_term_by_name(uc $ebi_interactor->uniprot_secondary);
 					my $obo_interactor_term = $ontology->get_term_by_xref('UniProt', $ebi_interactor->uniprot);
-# 					#&print_time;
 					if(defined $obo_interactor_term){
-# 						#&print_time;
-# 						print "Interactor already in OBO:",$ebi_interactor->uniprot_secondary,"\n";
 						&add_rel("participates_in",$obo_interactor_term,$OBOed_EBI_interaction,$ontology);
 						&add_rel("has_participant",$OBOed_EBI_interaction,$obo_interactor_term,$ontology);
-# 						#&print_time;
 					}
 					else{
-# 						#&print_time;
-# 						print "Adding interactor to OBO:",$ebi_interactor->shortLabel,"\n";
-						# Add the interactor term
-						my $OBOed_EBI_interactor = &add_interactor($ebi_interactor,$ontology,$cco_b_taxon_ids_map,$cco_b_ids_map);
-						# Add is_a
-# 						#&print_time;
-						&add_rel("participates_in",$OBOed_EBI_interactor,$OBOed_EBI_interaction,$ontology);	
-						&add_rel("has_participant",$OBOed_EBI_interaction,$OBOed_EBI_interactor,$ontology);
-# 						#&print_time;
+						my $role = &get_role ($ebi_interactor->id,$cco_intact_interaction);
+						# If good interactor is bait, include all the preys
+						if($good_interactor_is_bait==1 && $role eq "prey"){
+							# Add the interactor term
+							my $OBOed_EBI_interactor = &add_interactor($ebi_interactor,$ontology,$cco_b_taxon_ids_map,$cco_b_ids_map);
+							# Add is_a
+							&add_rel("participates_in",$OBOed_EBI_interactor,$OBOed_EBI_interaction,$ontology);	
+							&add_rel("has_participant",$OBOed_EBI_interaction,$OBOed_EBI_interactor,$ontology);
+						}
+						# If good interactor is prey, include only the bait
+						if($good_interactor_is_bait==0 && $role eq "bait"){
+							# Add the interactor term
+							my $OBOed_EBI_interactor = &add_interactor($ebi_interactor,$ontology,$cco_b_taxon_ids_map,$cco_b_ids_map);
+							# Add is_a
+							&add_rel("participates_in",$OBOed_EBI_interactor,$OBOed_EBI_interaction,$ontology);	
+							&add_rel("has_participant",$OBOed_EBI_interaction,$OBOed_EBI_interactor,$ontology);
+						}
+						if($role eq "neutral component"){
+							my $OBOed_EBI_interactor = &add_interactor($ebi_interactor,$ontology,$cco_b_taxon_ids_map,$cco_b_ids_map);
+							# Add is_a
+							&add_rel("participates_in",$OBOed_EBI_interactor,$OBOed_EBI_interaction,$ontology);	
+							&add_rel("has_participant",$OBOed_EBI_interaction,$OBOed_EBI_interactor,$ontology);
+						}					
 					}
 				}
 			}
@@ -176,32 +199,30 @@ sub work {
 	$cco_i_ids_map->write_map(); 
 	$cco_b_ids_map->write_map();
 	
-	#&print_time;
 	my $oboed_interactions = @oboed_interactions;
-# 	print $oboed_interactions," interactions added. \n";
  
 	# Compare the interactions already in OBO with the ones in the XML files:
 	# if there is an OBO interaction not present in XML file, delete it
-# 	#&print_time;
-# 	print pop(@all_interactions_EBI_ids),"\n";
-# 	print pop(@all_interactions_EBI_ids),"\n";
+	
+	# FAKE FOR TESTING 
+	#print pop(@all_interactions_EBI_ids),"\n";
+	#print pop(@all_interactions_EBI_ids),"\n";
+	# END FAKE FOR TESTING
 	my @obo_interactions = @{$ontology->get_terms("CCO:I.*")};
 	for my $obo_interaction (@obo_interactions){
 		my $obo_interaction_EBI_id = &get_xref_acc("IntAct",$obo_interaction);
 		if (defined $obo_interaction_EBI_id){
 			if(!defined &lookup($obo_interaction_EBI_id,\@all_interactions_EBI_ids)){
 				$ontology->delete_term($obo_interaction);
-# 				print "Deleting interaction from ontology:",$obo_interaction->name,"\n";
 			}
 		}
 	}
-# 	#&print_time;
 	
 	# Write the new ontology to disk
 	open (FH, ">".$intact_cco_obo_file_name) || die "Cannot write OBO file ", $!;
 	$ontology->export(\*FH);
 	close FH;
-	#&print_time;
+
 	return $ontology;
 }
 
@@ -215,58 +236,70 @@ sub lookup (){
 sub add_interactor(){
 	my ($ebi_interactor_object,$ontology,$cco_b_taxon_ids_map,$cco_b_ids_map) = @_;
 	my $interactor_name = uc $ebi_interactor_object->shortLabel;
-	my $interactor_cco_id = $cco_b_ids_map->get_new_cco_id("CCO", "B", $interactor_name);
-	my $OBOed_EBI_interactor = &add_term($interactor_name,$interactor_cco_id,$ontology);
-	$cco_b_taxon_ids_map->put($interactor_cco_id, $interactor_name); 
-	my $cco_protein=$ontology->get_term_by_name("protein"); 
-	#&print_time;
-	&add_rel("is_a", $OBOed_EBI_interactor, $cco_protein, $ontology);	
-	&add_xref($OBOed_EBI_interactor,"UniProt",$ebi_interactor_object->uniprot);
-	# Add def
-	if(defined $ebi_interactor_object->fullName){
-		#&print_time;
-		&add_def ($OBOed_EBI_interactor,"IntAct",$ebi_interactor_object->ebi_id,$ebi_interactor_object->fullName);
+	my $interactor_cco_id = $cco_b_taxon_ids_map->get_cco_id_by_term($interactor_name);
+	if (!defined $interactor_cco_id){
+		$interactor_cco_id = $cco_b_ids_map->get_new_cco_id("CCO", "B", $interactor_name);
+		$cco_b_taxon_ids_map->put($interactor_cco_id, $interactor_name); # TRICK to add the IDs in the other file
 	}
+	
+	my $OBOed_EBI_interactor = &add_term($interactor_name, $interactor_cco_id, $ontology);
+	my $cco_protein = $ontology->get_term_by_name("protein"); 
+
+	&add_rel("is_a", $OBOed_EBI_interactor, $cco_protein, $ontology);	
+	&add_xref($OBOed_EBI_interactor, "UniProt", $ebi_interactor_object->uniprot);
+	
+	my $ebi_interactor_object_ebi_id = $ebi_interactor_object->ebi_id();
+	#
+	# Add def
+	#
+	&add_def ($OBOed_EBI_interactor, "IntAct", $ebi_interactor_object_ebi_id, $ebi_interactor_object->fullName) if(defined $ebi_interactor_object->fullName);
+	#
 	# Add syn
+	#
 	for my $syn (@{$ebi_interactor_object->alias}){
-		#&print_time;
-		&add_synonym($OBOed_EBI_interactor,"IntAct",$ebi_interactor_object->ebi_id,$syn);
+		&add_synonym($OBOed_EBI_interactor, "IntAct", $ebi_interactor_object_ebi_id, $syn);
 	}
 	return $OBOed_EBI_interactor;
 }
 
 sub add_interaction(){
-	my ($good_interaction,$ontology,$cco_i_taxon_ids_map,$cco_i_ids_map) = @_;
-	my $interaction_id = $cco_i_ids_map->get_new_cco_id("CCO", "I",$good_interaction->shortLabel." ".$good_interaction->interactionType);
-	my $OBOed_EBI_interaction = &add_term($good_interaction->shortLabel." ".$good_interaction->interactionType,$interaction_id,$ontology);
-	$cco_i_taxon_ids_map->put($interaction_id, $good_interaction->shortLabel." ".$good_interaction->interactionType); 
-	# add is_a interaction
-	my $cco_interaction=$ontology->get_term_by_name("interaction"); 
-	#&print_time;
-	&add_rel("is_a", $OBOed_EBI_interaction, $cco_interaction, $ontology);
-	#&print_time;
-	# Add xref
-	&add_xref($OBOed_EBI_interaction,"IntAct",$good_interaction->primaryRef);
-	# Add def
-	if(defined $good_interaction->fullName){
-		#&print_time;
-		&add_def ($OBOed_EBI_interaction,"IntAct",$good_interaction->primaryRef,$good_interaction->fullName);
+	my ($good_interaction, $ontology, $cco_i_taxon_ids_map, $cco_i_ids_map) = @_;
+	
+	my $interaction = $good_interaction->shortLabel." ".$good_interaction->interactionType;
+	my $interaction_id = $cco_i_taxon_ids_map->get_cco_id_by_term($interaction);
+	if (!defined $interaction_id){
+		$interaction_id = $cco_i_ids_map->get_new_cco_id("CCO", "I", $interaction);
+		$cco_i_taxon_ids_map->put($interaction_id, $interaction); # TRICK to add the IDs in the other file
 	}
+	
+	my $OBOed_EBI_interaction = &add_term($interaction, $interaction_id, $ontology);
+	#
+	# Add is_a interaction
+	#
+# 	my $cco_interaction = $ontology->get_term_by_name("interaction"); 
+# 	&add_rel("is_a", $OBOed_EBI_interaction, $cco_interaction, $ontology);
+	
+	my $primary_ref = $good_interaction->primaryRef();
+	#
+	# Add xref
+	#
+	&add_xref($OBOed_EBI_interaction,"IntAct", $primary_ref);
+	#
+	# Add comment
+	#
+	&add_comment($OBOed_EBI_interaction,$good_interaction->fullName) if(defined $good_interaction->fullName());
 	return $OBOed_EBI_interaction;
 }
 
-# Given an interactor, it retrieves all the interactions that have the interactor
+# Given an interactor, retrieve all the interactions that have the interactor
 # as participant
 sub retrieve_interactions(){
 	my ($interactor,$interactions)=@_;
 	my @interactions=@{$interactions};
 	my @resulting_interactions=();
 	for my $interaction (@interactions){
-# 		print "CANDIDATE:",$interaction,"\n";
 		SYN: for my $syn (@{$interaction->interactorRef}){
-# 			print "SYN:",$syn,"-----",$interactor->id,"!\n";
 			if($syn eq $interactor->id){
-# 				print "GOOD INTERACTOR FOUND!!!", $interactor->id,"\n";
 				push(@resulting_interactions,$interaction);
 				last SYN;
 			}
@@ -290,9 +323,17 @@ sub merge_interactors {
 	return $interaction;
 }
 
-sub print_time{
-	my $now = localtime;
-	print $now,"\n";
+# Given and interactor id (e.g 498), get its role in the interaction
+sub get_role (){
+	my ($interactor_id,$interaction)=@_;
+	my %interRefROLES = %{$interaction->interactorRefRoles};
+	my $role = undef;
+	while ( my ($key, $value) = each(%interRefROLES) ) {
+		if($key eq $interactor_id){
+			$role=$value;
+		}
+    	}
+	return $role;
 }
 
 sub add_term (){
@@ -318,12 +359,6 @@ sub add_rel (){
 sub add_xref (){
 	my ($term,$db,$acc)=@_;
 	$term->xref_set_as_string( "[".$db.":".$acc."]");
-# 	my $xref = CCO::Core::Dbxref->new();
-# 	$xref -> db($db);
-# 	$xref -> acc($acc);
-# 	my $xref_set = CCO::Util::DbxrefSet->new();
-# 	$xref_set->add($xref);
-# 	$term->xref_set($xref_set);
 	return $term;
 }
 
@@ -333,15 +368,6 @@ sub add_def (){
 	$def_text =~ s/\t+//g;
 	$def_text =~ s/\r+//g;
 	$term->def_as_string($def_text, "[".$db.":".$acc."]");
-# 	my $def = CCO::Core::Def->new(); 
-# 	$def->text($def_text); 
-# 	my $dbxref_set = CCO::Util::DbxrefSet->new(); 
-# 	my $dbxref = CCO::Core::Dbxref->new(); 
-# 	$dbxref->db($db); 
-# 	$dbxref->acc($acc); 
-# 	$dbxref_set->add($dbxref); 
-# 	$def->dbxref_set($dbxref_set); 
-# 	$term -> def($def);
 	return $term; 
 }
 
@@ -351,34 +377,86 @@ sub add_synonym (){
 	return $term;  
 }
 
-sub check_term_exists_by_xref_acc(){
-	my ($unique_acc,@terms_to_check)=@_; 
-	my $result_term = undef;
-	for my $candidate_term (@terms_to_check){
-		my $dbxrefset = $candidate_term->xref_set;
-		XREF: for my $xref ($dbxrefset->get_set){
-			if($xref->acc eq $unique_acc){
-				$result_term = $candidate_term;
-				last XREF;
-			}
-		}
-	}
-	return $result_term;
+sub add_comment (){
+	my ($term, $text)=@_;
+	$text =~ s/\n+//g;
+	$text =~ s/\t+//g;
+	$text =~ s/\r+//g;
+	$term->comment($text);
+	return $term;  
 }
 
+# Given a term, get the xref of a given db
 sub get_xref_acc(){
 	my ($db,$term)=@_; 
 	my $result_acc = undef;
 	my $dbxrefset = $term->xref_set;
 	for my $xref ($dbxrefset->get_set){
 		if($xref->db eq $db){
-# 			print $xref->db,$db,"\n";
 			$result_acc = $xref->acc;		
 		}
 	}
 	return $result_acc;
 }
 
+sub generate_cco_y_ids(){
+	my %cco_y = (
+		'interaction type' => 'CCO:Y0000001',
+		'acetylation reaction'=> 'CCO:Y0000002',
+		'amidation reaction' => 'CCO:Y0000003',
+		'cleavage reaction' => 'CCO:Y0000004',	
+		'covalent binding' => 'CCO:Y0000005',	
+		'deacetylation reaction' => 'CCO:Y0000006',	
+		'defarnesylation reaction' => 'CCO:Y0000007',	
+		'deformylation reaction' => 'CCO:Y0000008',	
+		'degeranylation reaction' => 'CCO:Y0000009',	
+		'demyristoylation reaction' => 'CCO:Y0000010',	
+		'depalmitoylation reaction' => 'CCO:Y0000011',	
+		'dephosphorylation reaction' => 'CCO:Y0000012',	
+		'deubiquitination reaction' => 'CCO:Y0000013',	
+		'farnesylation reaction' => 'CCO:Y0000014',
+		'formylation reaction' => 'CCO:Y0000015',	
+		'genetic interaction' => 'CCO:Y0000016',
+		'geranylgeranylation reaction' => 'CCO:Y0000017',	
+		'hydroxylation reaction' => 'CCO:Y0000018',	
+		'lipid addition' => 'CCO:Y0000019',	
+		'lipid cleavage' => 'CCO:Y0000020',	
+		'methylation reaction' => 'CCO:Y0000021',	
+		'myristoylation reaction' => 'CCO:Y0000022',	
+		'palmitoylation reaction' => 'CCO:Y0000023',	
+		'phosphorylation reaction' => 'CCO:Y0000024',	
+		'physical interaction' => 'CCO:Y0000025',
+		'ubiquitination reaction' => 'CCO:Y0000026',	
+		'colocalization' => 'CCO:Y0000027',	
+		'direct interaction' => 'CCO:Y0000028',	
+		'disulfide bond' => 'CCO:Y0000029',	
+		'enzymatic reaction' => 'CCO:Y0000030',	
+		'transglutamination reaction' => 'CCO:Y0000031',	
+		'adp ribosylation reaction' => 'CCO:Y0000032',	
+		'deglycosylationreaction' => 'reaction CCO:Y0000033',	
+		'glycosylation reaction' => 'CCO:Y0000034',	
+		'sumoylation reaction' => 'CCO:Y0000035',	
+		'neddylation reaction' => 'CCO:Y0000036',	
+		'desumoylation reaction' => 'CCO:Y0000037',	
+		'deneddylation reaction' => 'CCO:Y0000038',	
+		'protein cleavage' => 'CCO:Y0000039',	
+		'mrna cleavage' => 'CCO:Y0000040',
+		'dna cleavage' => 'CCO:Y0000041',	
+		'dna strand elongation' => 'CCO:Y0000042',	
+		'synthetic interaction' => 'CCO:Y0000043',	
+		'asynthetic interaction' => 'CCO:Y0000044',	
+		'suppressive interaction' => 'CCO:Y0000045',	
+		'epistatic interaction' => 'CCO:Y0000046',	
+		'conditional interaction' => 'CCO:Y0000047',	
+		'additive interaction' => 'CCO:Y0000048',	
+		'single nonmonotonic interaction' => 'CCO:Y0000049',	
+		'double nonmonotonic' => 'interaction CCO:Y0000050',
+		'enhancement interaction' => 'CCO:Y0000051',	
+		'phosphotransfer reaction' => 'CCO:Y0000052'		
+	);
+	return \%cco_y;
+}
+
 1;
 
 =head1 NAME
